@@ -10,7 +10,7 @@ from matplotlib.ticker import StrMethodFormatter
 from casadi import Callback, nlpsol_out, nlpsol_n_out, Sparsity, DM
 
 from ..limits.path_conditions import Bounds
-from ..misc.enums import PlotType, ControlType, InterpolationType, Shooting
+from ..misc.enums import PlotType, ControlType, InterpolationType, Shooting, SolutionIntegrator
 from ..misc.mapping import Mapping
 from ..optimization.solution import Solution
 
@@ -197,7 +197,7 @@ class PlotOcp:
         automatically_organize: bool = True,
         show_bounds: bool = False,
         shooting_type: Shooting = Shooting.MULTIPLE,
-        use_scipy_integrator: bool = False,
+        integrator: SolutionIntegrator = SolutionIntegrator.DEFAULT,
     ):
         """
         Prepares the figures during the simulation
@@ -212,8 +212,9 @@ class PlotOcp:
             If the axes should fit the bounds (True) or the data (False)
         shooting_type: Shooting
             The type of integration method
-        use_scipy_integrator: bool
-            Use the scipy solve_ivp integrator for RungeKutta 45 instead of currently defined integrator
+        integrator: SolutionIntegrator
+             Use the ode defined by OCP or use a separate integrator provided by scipy
+
         """
         for i in range(1, ocp.n_phases):
             if len(ocp.nlp[0].states["q"]) != len(ocp.nlp[i].states["q"]):
@@ -235,7 +236,8 @@ class PlotOcp:
 
         self.t = []
         self.t_integrated = []
-        self.use_scipy_integrator = use_scipy_integrator
+        self.integrator = integrator
+
         if isinstance(self.ocp.original_phase_time, (int, float)):
             self.tf = [self.ocp.original_phase_time]
         else:
@@ -294,12 +296,14 @@ class PlotOcp:
         self.t_integrated = []
         last_t = 0
         for phase_idx, nlp in enumerate(self.ocp.nlp):
-            n_int_steps = nlp.ode_solver.steps_scipy if self.use_scipy_integrator else nlp.ode_solver.steps
+            n_int_steps = (
+                nlp.ode_solver.steps_scipy if self.integrator != SolutionIntegrator.DEFAULT else nlp.ode_solver.steps
+            )
             dt_ns = self.tf[phase_idx] / nlp.ns
             time_phase_integrated = []
             last_t_int = copy(last_t)
             for _ in range(nlp.ns):
-                if nlp.ode_solver.is_direct_collocation and not self.use_scipy_integrator:
+                if nlp.ode_solver.is_direct_collocation and self.integrator == SolutionIntegrator.DEFAULT:
                     time_phase_integrated.append(np.array(nlp.dynamics[0].step_time) * dt_ns + last_t_int)
                 else:
                     time_phase_integrated.append(np.linspace(last_t_int, last_t_int + dt_ns, n_int_steps + 1))
@@ -426,7 +430,11 @@ class PlotOcp:
                         )
                     elif plot_type == PlotType.INTEGRATED:
                         plots_integrated = []
-                        n_int_steps = nlp.ode_solver.steps_scipy if self.use_scipy_integrator else nlp.ode_solver.steps
+                        n_int_steps = (
+                            nlp.ode_solver.steps_scipy
+                            if self.integrator != SolutionIntegrator.DEFAULT
+                            else nlp.ode_solver.steps
+                        )
                         zero = np.zeros(n_int_steps + 1)
                         color = self.plot_func[variable][i].color if self.plot_func[variable][i].color else "tab:brown"
                         for cmp in range(nlp.ns):
@@ -585,7 +593,7 @@ class PlotOcp:
             continuous=False,
             shooting_type=self.shooting_type,
             keep_intermediate_points=True,
-            use_scipy_integrator=self.use_scipy_integrator,
+            integrator=self.integrator,
         ).states
         data_controls = sol.controls
         data_params = sol.parameters
@@ -598,7 +606,11 @@ class PlotOcp:
             self.__update_xdata()
 
         for i, nlp in enumerate(self.ocp.nlp):
-            step_size = nlp.ode_solver.steps_scipy + 1 if self.use_scipy_integrator else nlp.ode_solver.steps + 1
+            step_size = (
+                nlp.ode_solver.steps_scipy + 1
+                if self.integrator != SolutionIntegrator.DEFAULT
+                else nlp.ode_solver.steps + 1
+            )
             n_elements = nlp.ns * step_size + 1
 
             state = np.ndarray((0, n_elements))
